/**
 * Build a input with uiInputaction and uiAutocomplete widgets for system units handling/selection.
 *
 * The `ui.presentationUnitSelector()` build a input with uiInputaction and uiAutocomplete widgets for system units handling/selection. A hidden input is created and added to the DOM, so that the chosen value can be sent with a form.
 *
 * ### Examples
 * Create a input with uiInputaction and uiAutocomplete widgets for system units handling/selection.
 *
 * JavaScript:
 *
 *
 *  ep('#' + 'OrderUnitId').presentationUnitSelector({
 *      unitHidden: {
 *          name: "OrderUnit",
 *          value: #IF(#LocalOrderUnitId)"#LocalOrderUnitId"#ELSE#SPACE[0]false#ENDIF
 *      },
 *      bgClass: 'ep-ui-autocomplete-input-background',
 *      addClass: true
 *  });
 *
 * HTML:
 *
 *  <input type="text" class="ep-js ep-ui-autocomplete-input placeholder-input" id="OrderUnitId" name="OrderUnitId" value="" data-js=".uiValidate({'required': true})" />
 *
 *
 * @class jQuery.ui.presentationUnitSelector
 * @extends jQuery.widget
 *
 * @uses jQuery.ui.widget
 * @uses jQuery.ui.autocomplete
 * @uses ep
 * @uses de_epages
 * @uses de_epages.presentation.ui.inputaction
 * @since 7.88.0
 */

/**
 * Method toggles the background CSS class.
 *
 * @method handleBgClass
 * @member jQuery.ui.presentationUnitSelector
 *
 * @since 7.88.0
 */

/**
 * Method search for the currently chosen unit and return an object containing relevant data for the unit.
 *
 * @method getSelectedUnit
 * @member jQuery.ui.presentationUnitSelector
 *
 * @since 7.88.0
 */

/**
 * Method remove callback by name.
 *
 * @method removeCallback
 * @member jQuery.ui.presentationUnitSelector
 *
 * @since 7.88.0
 */

/**
 * See `jQuery.ui.presentationUnitSelector` for details.
 *
 * @param {Object} options A map of additional options pass to the method.
 *
 * @method presentationUnitSelector
 * @member jQuery
 *
 * @since 7.88.0
 */

/**
 * See `jQuery.ui.presentationUnitSelector` for details.
 *
 * @param {String} bgClass String of CSS classes for the input.
 *
 * @method presentationUnitSelector
 * @member jQuery
 *
 * @since 7.88.0
 */

/**
 * See `jQuery.ui.presentationUnitSelector` for details.
 *
 * @param {Boolean} addClass Indicates, if CSS class need to be added.
 *
 * @method presentationUnitSelector
 * @member jQuery
 *
 * @since 7.88.0
 */

/**
 * @cfg {Object} unitHidden An object containing data describing the hidden input.
 *                          This will be created and added to the DOM and is used to send data via a form.
 *                          (Structure see inside the commented part in option section in source code below)
 */

/**
 * @cfg {String} bgClass String of CSS classes for the input.
 */

/**
 * @cfg {Boolean} addClass Indicates, if CSS class need to be added.
 */

/*
 * @copyright       © Copyright 2006-2023, epages GmbH, All Rights Reserved.
 *
 * @module          ep.ui.presentationUnitSelector
 */

/*jslint nomen: true*/
/*global define, setTimeout*/
define("de_epages/presentation/ui/unit-selector", [
    "jquery",
    "ep",
    "de_epages",

    "jquery/ui/autocomplete",
    "de_epages/presentation/ui/inputaction",
    "jquery/ui/widget"
], function ($, ep) {
    'use strict';
    var html = $('html');

    $.widget('ui.presentationUnitSelector', {
    //    unitHidden,
    //    selectedUnit,
    //    units,

        options : {
        //    unitHidden: {name: "NameOfHiddenInput", value: "CurrentValueOrFalse"},
            bgClass: 'ep-ui-autocomplete-input-background',
            addClass: true
        },


        /*
         * Method called when the widget gets initialized. Used to configure basic data for the widget and
         * to create needed properties/elements and so on.
         *
         * @member jQuery.ui.presentationUnitSelector
         *
         * @since 7.88.0
         */
        _create : function () {
            var self = this,
                o = self.options,
                elem = self.element;

            self.units = ep.config.units ? ep.config.units : {units: []};

            if (o.unitHidden) {
                if (o.unitHidden.value) {
                    self.selectedUnit = self.getSelectedUnit();
                }

                if (o.unitHidden.name) {
                    // create hidden input for value transfer to server
                    self.unitHidden = $('<input type="hidden" id="'+ o.unitHidden.name +'" name="'+ o.unitHidden.name +'" />');
                    // add current selected unit ID
                    self.unitHidden.val(self.selectedUnit ? self.selectedUnit.value : '');
                    // append hidden input to document
                    elem.after(self.unitHidden);
                }
            }

            // a selected unit was handed over
            if (self.selectedUnit) {
                // set input values
                elem.val(self.selectedUnit.label);
                ep.changeSampleText(self.selectedUnit.label, o.unitHidden.name ? o.unitHidden.name : "Unit");
                // unitHidden.val(self.selectedUnit.value);
                o.addClass = false;
            }

            // set necessary css class
            self.handleBgClass(o.addClass);

            // add inputaction and autocomplete to input (elem)
            self._build();
        },


        /*
         * Method build the inputaction and autocompletion for the input and configures them.
         *
         * @member jQuery.ui.presentationUnitSelector
         *
         * @since 7.88.0
         */
        _build : function () {
            var self = this,
                elem = self.element,
                o = self.options;

            // initialize unit input
            elem
                // inputaction widget
                .presentationUiInputaction({
                    iconClass: 'fa-remove',
                    onAfterReset: function() {
                        self.handleBgClass(true);
                        self.unitHidden.val('');
                        ep.changeSampleText('', o.unitHidden ? o.unitHidden.name : 'unknown');
                        elem
                          .trigger('keyup.presentationUiInputaction')
                          .trigger('change');
                    }
                })
                // don't navigate away from the field on tab when selecting an item
                .on("keydown.unitSelector", function(event) {
                    if (event.keyCode === $.ui.keyCode.TAB && $(this).data('uiAutocomplete').menu.active) {
                        event.preventDefault();
                    }
                })
                .on("focus.unitSelector", function() {
                  var $this = $(this);

                    // open list/menu on focus event of input
                    $this.data("uiAutocomplete").search($this.val());
                })
                // autocomplete widget
                .autocomplete({
                    minLength: 0,
                    source: self.units.units,
                    classes: {
                      'ui-autocomplete': 'ep-ui-autocomplete-list ep-ui-autocomplete-max-height'
                    },

                    // define events
                    focus: function() {
                        // prevent value inserted on focus
                        return false;
                    },
                    response: function(event, ui) {
                        if (this.value.length === 0) {
                            self.handleBgClass(false);
                            ep.changeSampleText('', o.unitHidden ? o.unitHidden.name : 'unknown');
                        }
                    },
                    close: function(event, ui) {
                        if (this.value.length === 0) {
                            self.handleBgClass(true);
                            ep.changeSampleText('', o.unitHidden ? o.unitHidden.name : 'unknown');
                        }
                    },
                    // triggered if value in input changes
                    change: function(event, ui) {
                        // if currently chosen value is not in units-hash
                        if (!self.units.unitsHashByLabel[this.value]) {
                            // reset hidden input
                            self.unitHidden.val('');
                            ep.changeSampleText('', o.unitHidden ? o.unitHidden.name : 'unknown');
                        } else {
                            self.unitHidden.val(self.units.unitsHashByLabel[this.value]);
                            ep.changeSampleText(this.value, o.unitHidden ? o.unitHidden.name : 'unknown');
                        }

                        elem.trigger('keyup.presentationUiInputaction');
                    },
                    select: function(event, ui) {
                        // if currently chosen value changed
                        if (ui.item.value !== self.unitHidden.val()) {
                            // set unit name to input
                            this.value = ui.item.label;
                            // set unit id to hidden input
                            self.unitHidden.val(ui.item.value);
                            // set unit field
                            ep.changeSampleText(this.value, o.unitHidden ? o.unitHidden.name : 'unknown');

                            self.handleBgClass(false);
                            elem
                              .trigger('keyup.presentationUiInputaction')
                              .trigger('change');

                            if (o.onAfterSelect !== undefined && typeof o.onAfterSelect === "function") {
                                o.onAfterSelect();
                            }
                        }

                        return false;
                    }
                });
        },


        /*
         * Method can later be used to set further options (see comment for an example).
         *
         * @member jQuery.ui.presentationUnitSelector
         *
         * @since 7.88.0
         */
        _setOption : function (key, value) {
            var self = this,
                o = self.options;

            // switch (key) {
            // case 'zIndex':
            //     self.dialog.css('z-index', value);
            //     break;

            // }

            self._superApply(arguments);
        },


        /*
         * Method toggles the background CSS class.
         *
         * @member jQuery.ui.presentationUnitSelector
         *
         * @since 7.88.0
         */
        handleBgClass: function(add) {
            var self = this,
                o = self.options,
                elem = self.element;

            elem[add ? 'addClass' : 'removeClass'](o.bgClass);
        },


        /*
         * Method remove callback by name.
         *
         * @member jQuery.ui.presentationUnitSelector
         *
         * @since 7.88.0
         */
        removeCallback: function(callbackName) {
            var self = this,
                o = self.options;

            if(o[callbackName]) {
                delete o[callbackName];
            }
        },


        /*
         * Method search for the currently chosen unit and return an object containing relevant data for the unit.
         *
         * @member jQuery.ui.presentationUnitSelector
         *
         * @since 7.88.0
         */
        getSelectedUnit: function() {
            // filter units
            var self = this,
                o = self.options,
                selectedUnit = self.units.units.filter(function(item) {
                    return item.value === o.unitHidden.value;// "#OrderUnitId";
                });

            return (selectedUnit.length === 0) ? undefined : selectedUnit[0];
        },


        /*
         * Cleanup method to remove events, elements from DOM and destroy used widgets.
         *
         * @member jQuery.ui.presentationUnitSelector
         *
         * @since 7.88.0
         */
        destroy : function () {
            var self = this,
                elem = self.element;

            self._superApply(arguments);

            // remove events
            elem.off("keydown.unitSelector");
            elem.off("focus.unitSelector");

            // remove hidden input from DOM
            self.unitHidden.remove();

            // destroy used widgets
            elem.presentationUiInputaction('destroy');
            elem.autocomplete('destroy');
        }

    });

    return de_epages;
});
