/**
 * Display a price preview, including information for taxes, shipping and strike through price.
 *
 * The `.productUiPricePreview()` widget display a price preview for products. This includes information
 * for price, taxes, shipping and strike through price (RRP).
 *
 * ### Examples
 * Use .productUiPricePreview() to display a price preview.
 *
 * JavaScript:
 *
 *   $('.ep-pricepreview').productUiPricePreview({
 *       listPrice: 169.65,
 *       priceQuantity: 2,
 *       orderUnitId: "265",
 *       manufacturerPrice: 225,
 *       events: {
 *         listPrice: $('input[name="Price"]').eq(0),
 *         manufacturerPrice: $('input[name="ManufacturerPrice"]').eq(0),
 *         taxRate: $('#' + 'TaxClass'),
 *         isRRP: $('input[name="IsManufacturerPriceRRP"]'),
 *         shipping: $('input[name="IsShippingLinkVisible"]'),
 *         isVATInfoVisible: $('input[name="IsVATInfoVisible"]'),
 *         priceQuantity: $('#' + 'PriceQuantity'),
 *         orderUnit: $('#' + 'OrderUnitId')
 *       },
 *       texts: {
 *         vatReleasedMessage: "umsatzsteuerbefreit nach §19 UStG",
 *       },
 *       vatReleased: 1,
 *       isRRP: 1,
 *       shipping: 1,
 *       taxRate: 19,
 *       taxModel: 1, // 0 = without MwSt (net), 1 = with MwSt (gross)
 *       taxClass: true, // false = without tax, true = with tax
 *       isVATInfoVisible: 0,  // 1 = visible, 0 = hidden
 *       ref: {
 *          amount: 100,
 *          contentAmount: 1000,
 *          unit: "g"
 *       }
 *   });
 *
 * HTML:
 *
 *     <div class="ep-pricepreview"></div>
 *
 *
 * @class jQuery.ui.productUiPricePreview
 *
 * @uses ep
 * @uses de_epages
 * @uses de_epages.product.ui.pricepreview.dictionary
 * @uses jQuery.i18n
 * @uses jQuery.tmpl
 * @uses jQuery.ui.widget
 * @since 7.80.0
 */

/**
 * @cfg {Object} ref An object containing data reference unit (amount, contentAmount, unit).
 */

/**
 * @cfg {Float} listPrice List price of the product.
 */

/**
 * @cfg {Float} manufacturerPrice Manufacturer price of the product.
 */

/**
 * @cfg {Integer} vatReleased Indicates, if merchant is release from paying VAT (0=VAT, 1=released).
 */

/**
 * @cfg {Float} taxRate Tax rate.
 */

/**
 * @cfg {Integer} isRRP Indicates, if strike through pricing (0=no, 1=yes).
 */

/**
 * @cfg {Integer} shipping Indicates, if shipping costs are included (0=no, 1=yes).
 */

/**
 * @cfg {Integer} taxModel Indicates the tax model (0=net, 1=gross).
 */

/**
 * @cfg {Integer} inputTaxModel Indicates the tax model for price input in MBO (0=net, 1=gross).
 */

/**
 * @cfg {Boolean} taxClass Indicates, if with or without taxes (false=without taxes, true=with taxes).
 */

/**
 * @cfg {Object} events Contains nodes on which events will be registered.
 */

/**
 * @cfg {Object} texts Texts needed from outside the widget (vatReleasedMessage).
 */

/**
 * @cfg {Integer} isVATInfoVisible Indicates, if VAT info text is shown although tax rate is zero (0=no, 1=yes).
 */

/**
 * See `jQuery.ui.productUiPricePreview` for details.
 *
 * @param {Object} [ref] An object containing data reference unit (amount, contentAmount, unit).
 * @param {Float} listPrice List price of the product.
 * @param {Float} manufacturerPrice Manufacturer price of the product.
 * @param {Integer} vatReleased Indicates, if merchant is release from paying VAT (0=VAT, 1=released).
 * @param {Float} taxRate Tax rate.
 * @param {Integer} isRRP Indicates, if strike through pricing (0=no, 1=yes).
 * @param {Integer} shipping Indicates, if shipping costs are included (0=no, 1=yes).
 * @param {Integer} taxModel Indicates the tax model (0=net, 1=gross).
 * @param {Boolean} taxClass Indicates, if with or without taxes (false=without taxes, true=with taxes).
 * @param {Object} events Contains nodes on which events will be registered.
 * @param {Object} texts Texts needed from outside the widget (vatReleasedMessage).
 * @param {Integer} isVATInfoVisible Indicates, if VAT info text is shown although tax rate is zero (0=no, 1=yes).
 *
 * @method productUiPricePreview
 * @member jQuery
 *
 * @since 7.80.0
 */

/*
 * @copyright       © Copyright 2006-2023, epages GmbH, All Rights Reserved.
 *
 * @module          de_epages/product/ui/pricepreview
 */

define("de_epages/product/ui/pricepreview", [
    "jquery",
    "ep",
    'de_epages',
    "$tmpl!./pricepreview",
    "$dict!../dictionary",

    "jquery/i18n",
    "jquery/tmpl",
    "jquery/ui/widget"
], function ($, ep, de_epages, tmplPricepreview, productDict) {

    /*
     * @dictionary       de_epages.product.dictionary
     *
     * @translation      {IsGross}
     *                   {IsNet}
     *                   {PriceManufacturer}
     *                   {PricePreviewHeadline}
     *                   {WithoutShipping}
     */

    /*
     * @dictionary      ep.dict
     */

    var translations = {
        priceManufacturer: productDict.translate('PriceManufacturer'),
        pricepreviewHeadline: productDict.translate('PricePreviewHeadline'),
        isGross: productDict.translate('IsGross'),
        isNet: productDict.translate('IsNet'),
        withoutShipping: productDict.translate('WithoutShipping')
    };

    $.widget('ui.productUiPricePreview', {
        // default options
        options: {
        //  ref: {amount: null, contentAmount: null, unit: null},
        //  orderUnit: {value, label, abbr, alias, show},
        //  taxRate: 0,
            units: [],
            listPrice: null,
            priceQuantity: null,
            orderUnitId: null,
            manufacturerPrice: null,
            vatReleased: 0,
            isRRP: 1,    // 1 = RRP, 0 = without RRP
            shipping: 1, // 1 = with shipping, 0 = without
            taxModel: 1, // 1 = gross, 0 = net
            inputTaxModel: 1, // 1 = gross, 0 = net
            taxClass: 0,
            isVATInfoVisible: 1, // 1 = visible, 0 = hidden
            events: {
                // listPrice
                // manufacturerPrice
                // taxRate
                // isRRP
                // shipping
                // isVATInfoVisible
                // refAmount
                // refUnit
                // refContentAmount
                // priceQuantity
                // orderUnit
            },
            texts: {
                vatReleasedMessage: ""
            }
        },

        _create: function() {
            var self = this,
                o = self.options,
                tmpListPrice,
                tmpManufacturerPrice;

            // merge translations with texts from options
            $.extend(translations, o.texts);

            // if oderUnitId option is set
            if (o.orderUnitId !== null) {
                o.orderUnit = self._findUnitById(o.orderUnitId)[0];
            }

            self._registerEvents();
            self._render();
        },

        /*
         * Method calculates percentage difference between list and manufacturer price.
         *
         * @member jQuery.ui.productUiPricePreview
         *
         * @since 7.80.0
         */
        _calcPercent: function(listPrice, manufacturerPrice) {
            return !!manufacturerPrice ? -1 * (100 - (100 * listPrice / manufacturerPrice)) : 0;
        },

        /*
         * Method calculates prices with correct taxes.
         *
         * @member jQuery.ui.productUiPricePreview
         *
         * @since 7.84.0 (UNITY-8428)
         */
        _calcPrice: function(price) {
            var self = this,
                o = self.options,
                tmpPrice = price;

            if (o.taxRate === undefined) return tmpPrice;

            // if shop tax model and input tax model differ => recalculate price
            if (o.taxModel !== o.inputTaxModel) {
                // gross
                if (o.taxModel === 1) {
                    tmpPrice = price * (100 + o.taxRate) / 100;
                // net
                } else {
                    tmpPrice = price * 100 / (100 + o.taxRate);
                }
            }

            return tmpPrice;
        },

        /*
         * Method formats plain number to currency depending on the users region.
         *
         * @member jQuery.ui.productUiPricePreview
         *
         * @since 7.80.0
         */
        _formatCurrency: function(number) {
            return $.i18n.formatNumber(
                number,
                'c2',
                {
                    region: $.i18n.settings.region,
                    currency: $.i18n.settings.currency
                }
            );

        },

        /*
         * Method to update options (= array with name/value objects).
         *
         * @member jQuery.ui.productUiPricePreview
         *
         * @since 7.80.0
         */
        _updateOptions: function(options) {
            var self = this,
                o = self.options;

            options.forEach(function(option) {
                o[option.name] = option.value;
            });
        },

        /*
         * Filters units array by a given label name (= unit name).
         *
         * @member jQuery.ui.productUiPricePreview
         *
         * @since 7.80.0
         */
        _findUnitByLabel: function(label) {
            var self = this,
                o = self.options;

            return o.units.filter(function(unit) {
                return unit.label === label;
            });
        },

        /*
         * Filters units array by a given unit id
         *
         * @member jQuery.ui.productUiPricePreview
         *
         * @since 7.89.0
         */
        _findUnitById: function(id) {
            var self = this,
                o = self.options;

            return o.units.filter(function(unit) {
                return unit.value === id;
            });
        },

        /*
         * Method to registern events.
         *
         * @member jQuery.ui.productUiPricePreview
         *
         * @since 7.80.0
         */
        _registerEvents: function() {
            var self = this,
                o = self.options;

            // list price
            o.events.listPrice && o.events.listPrice.on('input.previewInput', function() {
                self._updateOptions([{
                    name: 'listPrice',
                    value: $.i18n.parseNumber($(this).val())
                }]);
                self._render();
            });
            // manufacturer price
            o.events.manufacturerPrice && o.events.manufacturerPrice.on('input.previewInput', function() {
                self._updateOptions([{
                    name: 'manufacturerPrice',
                    value: $.i18n.parseNumber($(this).val())
                }]);
                self._render();
            });
            // current tax rate
            o.events.taxRate && o.events.taxRate.on('change.previewInput', function() {
                var newTaxRate = $(this).find('option').eq(this.selectedIndex).data("calculatedTaxrate");

                self._updateOptions([{
                    name: 'taxRate',
                    value: newTaxRate
                }]);
                self._render();
            });
            // RRP
            o.events.isRRP && o.events.isRRP.each(function(index, input) {
                $(input).on('change.previewInput', function() {
                    self._updateOptions([{
                        name: 'isRRP',
                        value: $.i18n.parseNumber($(this).val())
                    }]);
                    self._render();
                });
            });
            // shipping link
            o.events.shipping && o.events.shipping.each(function(index, input) {
                $(input).on('change.previewInput', function() {
                    self._updateOptions([{
                        name: 'shipping',
                        value: $.i18n.parseNumber($(this).val())
                    }]);
                    self._render();
                });
            });
            // is VAT info visible info text
            o.events.isVATInfoVisible && o.events.isVATInfoVisible.each(function(index, input) {
                $(input).on('change.previewInput', function() {
                    self._updateOptions([{
                        name: 'isVATInfoVisible',
                        value: $.i18n.parseNumber($(this).val())
                    }]);
                    self._render();
                });
            });
            // reference amount
            o.events.refAmount && o.events.refAmount.on('input.previewInput', function() {
                o.ref = o.ref || {};
                $.extend(o.ref, {
                    amount: $.i18n.parseNumber($(this).val())
                });
                self._render();
            });
            // reference content amount
            o.events.refContentAmount && o.events.refContentAmount.on('input.previewInput', function() {
                o.ref = o.ref || {};
                $.extend(o.ref, {
                    contentAmount: $.i18n.parseNumber($(this).val())
                });
                self._render();
            });
            // unit
            o.events.refUnit && o.events.refUnit.on('change.previewInput', function() {
                var unit = self._findUnitByLabel($(this).val());

                unit = unit.length ? unit[0].abbr : null;
                o.ref = o.ref || {};
                $.extend(o.ref, {
                    unit: unit
                });
                self._render();
            });
            // price quantity
            o.events.priceQuantity && o.events.priceQuantity.on('input.previewInput', function() {
                self._updateOptions([{
                    name: 'priceQuantity',
                    value: $.i18n.parseNumber($(this).val())
                }]);
                self._render();
            });
            // order unit
            o.events.orderUnit && o.events.orderUnit.watch && o.events.orderUnit.watch.on('change.previewInput', function() {
                var unit = self._findUnitById(o.events.orderUnit.value.val())[0];

                self._updateOptions([{
                    name: 'orderUnit',
                    value: unit
                }]);
                self._render();
            });
        },

        /*
         * Method renders the template.
         *
         * @member jQuery.ui.productUiPricePreview
         *
         * @since 7.80.0
         */
        _render: function() {
            var self = this,
                o = self.options,
                elem = self.element,
                // recalc prices depending on taxes (net/gross)
                listPrice = self._calcPrice(o.listPrice),
                manufacturerPrice = self._calcPrice(o.manufacturerPrice),

                renderedTmpl;

            renderedTmpl = tmplPricepreview({
                manufacturerPrice: manufacturerPrice ? self._formatCurrency(manufacturerPrice) : undefined,
                listPrice: listPrice ? self._formatCurrency(listPrice) : undefined,
                priceQuantity: o.priceQuantity
                    ? $.i18n.formatNumber(
                        o.priceQuantity,
                        'n' + (Number.isInteger(o.priceQuantity) ? '0' : '2'),
                        {
                            region: $.i18n.settings.region,
                            currency: $.i18n.settings.currency
                        })
                    : undefined,
                showPriceQuantity: o.priceQuantity ? (o.priceQuantity !== 1) : false,
                orderUnit: o.orderUnit ? o.orderUnit : undefined,
                pricePercent: Math.round(self._calcPercent(listPrice, manufacturerPrice)),
                vatReleased: o.vatReleased,
                ref: (o.ref && (o.ref.amount !== null && o.ref.amount !== 0) && o.ref.unit !== null && o.ref.contentAmount !== null && o.ref.contentAmount !== 0)
                     ? $.extend({}, o.ref, {
                          price: self._formatCurrency(listPrice * o.ref.amount / o.ref.contentAmount)
                       })
                     : undefined,
                taxRate: o.taxRate,
                isVATInfoVisible: o.isVATInfoVisible,
                isRRP: o.isRRP,
                shipping: o.shipping,
                taxModel: o.taxModel,
                inputTaxModel: o.inputTaxModel,
                taxClass: o.taxClass,
                t: translations
            });

            elem.html(renderedTmpl);
        },

        /*
         * Method to destroy the widget.
         * - remove events
         * - reset HTML
         *
         * @member jQuery.ui.productUiPricePreview
         *
         * @since 7.80.0
         */
        destroy: function () {
            var self = this,
                elem = self.element,
                o = self.options;

            self._superApply(arguments);

            // remove events
            o.events.listPrice && o.events.listPrice.off('input.previewInput');
            o.events.priceQuantity && o.events.priceQuantity.off('input.previewInput');
            o.events.orderUnit && o.events.orderUnit.watch && o.events.orderUnit.watch.off('change.previewInput');
            o.events.manufacturerPrice && o.events.manufacturerPrice.off('input.previewInput');
            o.events.taxRate && o.events.taxRate.off('change.previewInput');
            o.events.refAmount && o.events.refAmount.off('input.previewInput');
            o.events.refContentAmount && o.events.refContentAmount.off('input.previewInput');
            o.events.refUnit && o.events.refUnit.off('change.previewInput');
            o.events.isRRP && o.events.isRRP.each(function(index, input) {
                $(input).off('change.previewInput');
            });
            o.events.shipping && o.events.shipping.each(function(index, input) {
                $(input).off('change.previewInput');
            });
            o.events.isVATInfoVisible && o.events.isVATInfoVisible.each(function(index, input) {
                $(input).off('change.previewInput');
            });

            // remove html
            elem.find('*').remove();
        }
    });

    return de_epages;
});
